-- Spawns a new sprite
function newSprite(x, y, textureID)
	table.insert(sprites, {x = x, y = y, id = textureID})
end

-- Spawns a new door
function newDoor(x, y, textureID, wide, mapX, mapY) -- Wide determines whether or not the door is horizontal or not
	table.insert(doors, {x = x, y = y, id = textureID, wide = wide, open = false, opening = false, mapX = mapX, mapY = mapY, movementLeft = 1,})
end

-- Checks for a door
function checkDoor(mapX, mapY)
	for i=1, #doors do
		if doors[i].mapX == mapX and doors[i].mapY == mapY then
			if doors[i].open then
				return true
			else
				doors[i].opening = true
				return false
			end
		end
	end

	return false
end

-- Updates Doors
function updateDoor(dt)
	for i=1, #doors do
		if doors[i].opening then
			if doors[i].wide then
				doors[i].y = doors[i].y - (dt / 2)
			else
				doors[i].x = doors[i].x - (dt / 2)
			end
			doors[i].movementLeft = doors[i].movementLeft - (dt / 2)

			if doors[i].movementLeft <= 0 then
				doors[i].opening = false
				doors[i].open = true
			end
		end
	end
end

-- Gets the rotation of the player
function getPlayerRot()
	return math.atan2(player.dirX, player.dirY)
end

-- Stencil
function stencil()
	if beginSt < endSt then
		love.graphics.rectangle("fill", beginSt, 0, endSt - beginSt, h)
	else
		love.graphics.rectangle("fill", endSt, 0, beginSt - endSt, h)
	end
end

-- Sort Algorithm
function zSort(object1, object2)
    return object1.z > object2.z
end

-- Player Input
function input(dt)
	local moveSpeed = dt * player.moveMod
	local rotSpeed = dt * player.rotMod
	local strafeSpeed = dt * player.strafeMod

	if love.keyboard.isDown("w") then
		local mapX = math.floor(player.posX + player.dirX * moveSpeed)
		local mapY = math.floor(player.posY)
		local tempMove = map[mapX][mapY]
		if (tempMove == 0 or ((tempMove == -7 or tempMove == -8) and checkDoor(mapX, mapY))) then
			player.posX = player.posX + player.dirX * moveSpeed
		end

		local mapX = math.floor(player.posX)
		local mapY = math.floor(player.posY + player.dirY * moveSpeed)
		local tempMove = map[mapX][mapY]
		if (tempMove == 0 or ((tempMove == -7 or tempMove == -8)  and checkDoor(mapX, mapY))) then
			player.posY = player.posY + player.dirY * moveSpeed
		end
	end
	if love.keyboard.isDown("s") then
		local mapX = math.floor(player.posX - player.dirX * moveSpeed)
		local mapY = math.floor(player.posY)
		local tempMove = map[mapX][mapY]
		if (tempMove == 0 or ((tempMove == -7 or tempMove == -8)  and checkDoor(mapX, mapY))) then
			player.posX = player.posX - player.dirX * moveSpeed
		end

		local mapX = math.floor(player.posX)
		local mapY = math.floor(player.posY - player.dirY * moveSpeed)
		local tempMove = map[mapX][mapY]
		if (tempMove == 0 or ((tempMove == -7 or tempMove == -8)  and checkDoor(mapX, mapY))) then
			player.posY = player.posY - player.dirY * moveSpeed
		end
	end
	if love.keyboard.isDown("d") then
		local mapX = math.floor(player.posX + player.planeX * moveSpeed)
		local mapY = math.floor(player.posY)
		local tempMove = map[mapX][mapY]
		if (tempMove == 0 or ((tempMove == -7 or tempMove == -8)  and checkDoor(mapX, mapY))) then
			player.posX = player.posX + player.planeX * strafeSpeed
		end

		local mapX = math.floor(player.posX)
		local mapY = math.floor(player.posY + player.planeY * moveSpeed)
		local tempMove = map[mapX][mapY]
		if (tempMove == 0 or ((tempMove == -7 or tempMove == -8)  and checkDoor(mapX, mapY))) then
			player.posY = player.posY + player.planeY * strafeSpeed
		end
	end
	if love.keyboard.isDown("a") then
		local mapX = math.floor(player.posX - player.planeX * moveSpeed)
		local mapY = math.floor(player.posY)
		local tempMove = map[mapX][mapY]
		if (tempMove == 0 or ((tempMove == -7 or tempMove == -8)  and checkDoor(mapX, mapY))) then
			player.posX = player.posX - player.planeX * strafeSpeed
		end

		local mapX = math.floor(player.posX)
		local mapY = math.floor(player.posY - player.planeY * moveSpeed)
		local tempMove = map[mapX][mapY]
		if (tempMove == 0 or ((tempMove == -7 or tempMove == -8)  and checkDoor(mapX, mapY))) then
			player.posY = player.posY - player.planeY * strafeSpeed
		end 
	end

	if grabbed then
		local mouseX = love.mouse.getX()
		local mouseY = love.mouse.getY()

		if mouseX ~= w/2 then
			local turnSpeed = 0
			if mouseX > w/2 then turnSpeed = rotSpeed * (w/2 - mouseX) end
			if mouseX < w/2 then turnSpeed = -rotSpeed * (mouseX - (w/2)) end

			local oldDirX = player.dirX
			player.dirX = player.dirX * math.cos(turnSpeed) - player.dirY * math.sin(turnSpeed)
			player.dirY = oldDirX * math.sin(turnSpeed) + player.dirY * math.cos(turnSpeed)
			local oldPlaneX = player.planeX
			player.planeX = player.planeX * math.cos(turnSpeed) - player.planeY * math.sin(turnSpeed)
			player.planeY = oldPlaneX * math.sin(turnSpeed) + player.planeY * math.cos(turnSpeed)
		end
		
		-- Head Tilt (Not Working)
		--[[ if mouseY ~= h/2 then
			local tiltSpeed = 0
			if mouseY > h/2 then tiltSpeed = 400 * (h/2 - mouseY) end
			if mouseY < h/2 then tiltSpeed = -400 * (mouseY - (h/2)) end

			if TILT + tiltSpeed * dt > -tLim and TILT + tiltSpeed * dt < tLim then
				TILT = TILT + tiltSpeed * dt
			end
		end ]]--

		love.mouse.setPosition(w/2, h/2)
	end
end